<?php

/**
 * Creates fields that can be saved to content types.
 *
 * @code
 * $my_field = CINC::init('Field')->set('field_name', 'my_field');
 * @endcode
 */
class CINC_Field extends CINC implements CINC_CRUD {

  public $config_machine_name = 'field_name';
  public static $cinc_yaml_regex = '/^field\.storage\./';
  public $instance = NULL;

  function __construct() {

    $this->config_filters['field_name'] = 'set_field_name';

    $this->config = array(
      'field_name' => '',
      'active' => 1,
      'locked' => 0,
      'cardinality' => 1,
      'deleted' => 0,
    );

    $this->instance = CINC::init('FieldInstance');

  }

  /**
   * Auto-adds label when field_name is set, and cleans field_name.
   */
  public function set_field_name(&$field_name) {
    $field_name = CINC::clean_machine_name($field_name);
    if (empty($this->instance_config['label'])) {
      $this->set_instance('label', CINC::display_name_from_machine_name($field_name));
    }
    if (empty($this->instance_config['field_name'])) {
      $this->set_instance('field_name', $field_name);
    }
    return TRUE;
  }

  /**
   * Sets properties of the field instance.
   *
   * @code
   * CINC::init('Field')
   *   ->set('field_name', 'field_my_field')
   *   ->set_instance('settings', array('user_register_form' => TRUE));
   * @endcode
   *
   * @param string $property
   *   Property to set.
   * @param string|array $value
   *   Value to set on property.
   * @return $this
   */
  public function set_instance($property, $value) {
    $this->instance->set($property, $value);
    return $this;
  }

  /**
   * Saves field information to the database.
   */
  public function create() {
    field_create_field($this->config);

    return $this;
  }

  /**
   * Loads field information from the database.
   */
  public function read() {
    $this->config = field_read_field($this->config['field_name'], array('include_inactive' => TRUE));
    return $this;
  }

  /**
   * Loads field instance information from the database.
   */
  public function read_instance_for($entity_type, $bundle) {
    $this->instance->set('entity_type', $entity_type);
    $this->instance->set('bundle', $bundle);
    $this->instance->read();
    return $this;
  }

  /**
   * Saves field information to the database and migrates previous data if machine name changed.
   */
  public function update() {
    field_update_field($this->config);
    return $this;
  }

  /**
   * Removes field information from the database.
   */
  public function delete() {
    field_delete_field($this->config['field_name']);
    return $this;
  }

  public function delete_and_purge() {
    $this->delete();
    field_purge_batch(10);
  }

  /**
   * Saves a field to an entity bundle.
   *
   * @code
   * $field = CINC::init('Field')->set('field_name', 'my_field')->create();
   * $field->create_instance_for('node', 'my_content_type');
   * @endcode
   *
   * @param string $entity_type
   * @param string $bundle
   * @param integer $weight
   */
  public function create_instance_for($entity_type, $bundle, $weight = NULL) {

    $this->instance->set('entity_type', $entity_type);
    $this->instance->set('bundle', $bundle);
    if ($weight !== NULL) {
      $this->instance->set('widget', array('weight' => $weight));
    }

    $this->instance->create();

    return $this;
  }

}

/**
 * Creates field instancess that can be saved to content types.
 *
 * @code
 * $my_field_instance = CINC::init('FieldInstance')->machine_name('field_my_field');
 * @endcode
 */
class CINC_FieldInstance extends CINC implements CINC_CRUD {

  public $config_machine_name = 'field_name';
  public static $cinc_yaml_regex = '/^field\.instance\./';

  function __construct() {

    $this->config = array(
      'label' => '',
      'required' => 0,
      'description' => '',
      'field_name' => '',
      'entity_type' => 'node',
      'bundle' => '',
      'deleted' => 0,
      'settings' => array(
        'user_register_form' => FALSE,
      ),
      'widget' => array(
        'active' => 1,
      ),
    );

  }

  /**
   * Saves field instance information to the database.
   */
  public function create() {
    field_create_instance($this->config);
    return $this;
  }

  /**
   * Loads field instance information from the database.
   */
  public function read() {
    $this->config = field_read_instance($this->config['entity_type'], $this->config['field_name'], $this->config['bundle']);;
    return $this;
  }

  /**
   * Saves field instance information to the database.
   */
  public function update() {
    $this->create();
    return $this;
  }

  /**
   * Removes field instance information from the database.
   */
  public function delete() {
    field_delete_instance($this->config);
    return $this;
  }

}
